module Labyrinth where
import VTree

-- Modelling labyrinths

-- An identifier for the nodes:
type Id  = Integer

-- The Labyrinth as a variadic tree:
type Lab = VTree Id

-- A path in the labyrinth is now a list of node ids:
type Path = [Id]

-- A traversal: a successful path, or a failure:
type Trav = Maybe Path

-- Add two traversals, disregarding first if it fails:
select :: Trav-> Trav-> Trav
select Nothing t = t
select t       _ = t

-- Traversal of a labyrinth (with cycles) to a given node:
traverse2 :: Id-> Lab-> Path-> Trav
traverse2 t (VNode i l) p
  | i == t    = Just (reverse (i: p))
  | elem i p  = Nothing
  | otherwise = travChildren l where
    travChildren []     = Nothing 
    travChildren (l:ls) = 
        select (traverse2 t l (i: p)) (travChildren ls)


-- An example labyrinth with cycles:
m00 = VNode 0 [m01]
m10 = VNode 1 [m00]
m20 = VNode 2 [m21]
m30 = VNode 3 [m20]
m40 = VNode 4 []
m01 = VNode 5 [m02] -- Alternatively, VNode 5 [m11, m02]
m11 = VNode 6 [m01] -- This makes one cycle
m21 = VNode 7 [m11]
m31 = VNode 8 [m30, m41]
m41 = VNode 9 [m40, m42]
m02 = VNode 10 [m12, m03]
m12 = VNode 11 [m13]
m22 = VNode 12 [m32]
m32 = VNode 13 [m31]
m42 = VNode 14 []
m03 = VNode 15 [m04]
m13 = VNode 16 [m23]
m23 = VNode 17 [m22, m24]
m33 = VNode 18 [m43]
m43 = VNode 19 [m44]
m04 = VNode 20 [m14]
m14 = VNode 21 [m24] -- This makes the other cycle
m24 = VNode 22 [m34] -- Alternatively, VNode 22 [m14, m34]
m34 = VNode 23 [m33]
m44 = VNode 24 []
